# encoding: utf-8
"""
@project: sky-excel->excel_exporter
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: excel输出者
@created_time: 2022/9/29 17:01
"""
from io import BytesIO
import os
import platform
import re
import uuid

import xlrd
from xlutils.copy import copy
from xlwt import Workbook

from utils.j_dict import JDict


def catch_except(func):
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as e:
            err_detail = " file:" + str(e.__traceback__.tb_frame.f_globals["__file__"]) + " line:" + str(e.__traceback__.tb_lineno) if platform.python_version()[0] == "3" else ""
            return None, "系统异常：" + str(e) + err_detail

    return wrapper


class ExcelExport:
    __err_msg = None
    max_row_resolve = []  # 导出模板的最后一行单元格内容列表。是导出变量的key值。
    max_row_resolve_map = {}  # 最后一行单元格解析,映射字典。
    max_row_merged_start_map = {}  # 追后一行合并的单元格解析,映射字典。注意合并的单元格包含普通单元格，数据是存放合并单元格的左上角单元格中。
    is_additional_write = True  # 是否可以追加读写。True：代表加载到excel的导出模板；False:表示没有找到excel模板所以生成新的excel文件

    @property
    def err_msg(self):
        return self.__err_msg

    @err_msg.setter
    def err_msg(self, err):
        # 返回最先暴出来的错误，按照顺序返回，避免覆盖错误无法排查
        if self.__err_msg is None:
            self.__err_msg = err

    def __init__(self, excel_templet_path=None):
        """
        :param excel_templet_path: 模板的路径，可用则追加读写方式，不可用只写模式
        """
        if excel_templet_path and os.path.exists(excel_templet_path):
            # 追加读写方式
            self.is_additional_write = True
            self.read_wb = xlrd.open_workbook(excel_templet_path, formatting_info=True)  # 直接解析模板，获取可操作的excel对象
            self.additional_write_wb = copy(wb=self.read_wb)  # 完成xlrd对象向xlwt对象转换
            self.sheet_names = self.read_wb.sheet_names()
        else:
            # 仅写模式
            self.is_additional_write = False

        # 完成xlrd对象向xlwt对象转换
        self.only_write_wb = Workbook()

    def parse_max_row(self, excel_templet_title: str = "Sheet1"):
        """
        默认模板规范：excel 模板最后一行填写数据变量映射使用{{}}包裹
        得到结果：[0: {'col_coord': (0, 1), 'value': '{{col_1}}'},....]。 col_coord表示列表坐标区间，value表示变量的值
        :param excel_templet_title: 在追加读写模式下，指定读取的sheet，默认Sheet1
        :return: (data,err); data：{0: {'col_coord': (0, 1), 'value': '{{col_1}}'},....}
        """
        # 找到可操作的工作簿和页
        if not self.is_additional_write:
            return None, "您传的模板路径不正确，所以不可以进行解析操作"
        if not excel_templet_title in self.sheet_names:
            return None, "不存在名称为：%s 的工作簿" % excel_templet_title

        read_sheet = self.read_wb.sheet_by_name(excel_templet_title)
        # 最后一行合并单元格解析
        self.max_row_merged_start_map = {
            start_cell: (start_cell, end_cell) for start_row, end_row, start_cell, end_cell in read_sheet.merged_cells if end_row == read_sheet.nrows
        }
        try:
            # 获取表达式的注意 merged_cells 坐标从0开始，max_row从1开始，merged_cells包括开始不包括结束;
            self.max_row_resolve = []
            self.max_row_resolve_map = {}
            start_merge = end_merge = -1
            for index in range(read_sheet.row_len(read_sheet.nrows - 1)):
                # 因为excel合并单元格数据存放在左上角的单元格之中,其他单元格不存放数据。
                if start_merge <= index < end_merge:
                    continue
                cell = JDict({})
                if self.max_row_merged_start_map.get(index):
                    start_merge, end_merge = self.max_row_merged_start_map.get(index)
                    cell["col_coord"] = (index, end_merge)
                else:
                    cell["col_coord"] = (index, index + 1)
                cell["value"] = read_sheet.cell(read_sheet.nrows - 1, index).value

                # 数据填充
                self.max_row_resolve.append(cell)
                self.max_row_resolve_map.update({index: cell})
            return self.max_row_resolve_map, None
        except Exception as e:
            err_detail = " file:" + str(e.__traceback__.tb_frame.f_globals["__file__"]) + " line:" + str(e.__traceback__.tb_lineno) if platform.python_version()[0] == "3" else ""
            return None, "系统异常：" + str(e) + err_detail

    # 追加写入body
    def additional_write(self, input_dict, excel_templet_title="Sheet1"):
        """
        追加读写
        :param input_dict: [{..},{..}..]
        :param excel_templet_title: 操作的sheet,需要与parse_max_row保持一致
        :return: data,err
        """
        # 合法性判断
        if not self.is_additional_write:
            return None, "当前不是追加读写模式，请检查模板路径是否正确"
        if not excel_templet_title in self.sheet_names:
            return None, "不存在名称为：%s 的工作簿" % excel_templet_title
        if not isinstance(input_dict, list):
            self.err_msg = "input_dict格式应该是[{..},{..}..]"
        if not self.max_row_resolve:
            data, err = self.parse_max_row()
            if err:
                return None, err

        # 初始化
        read_sheet = self.read_wb.sheet_by_name(excel_templet_title)
        write_sheet = self.additional_write_wb.get_sheet(excel_templet_title)
        max_row = read_sheet.nrows  # 最大行
        max_col_num = read_sheet.row_len(max_row - 1)  # 最大列
        for row_num, row_input_dict in zip(range(max_row - 1, len(input_dict) + max_row), input_dict):
            # 行遍历，从最后一行开始。
            start_merge_cell = end_merge_cell = -1
            for col_index in range(max_col_num):  # 列遍历
                if start_merge_cell <= col_index < end_merge_cell:  # 被合并单元格，跳过
                    continue
                this_cell_value = self.parse_expression(self.max_row_resolve_map.get(col_index).value, row_input_dict)
                # 写入数据，单个单元格写入与合并单元表格写入
                if self.max_row_merged_start_map.get(col_index):
                    start_merge_cell, end_merge_cell = self.max_row_merged_start_map.get(col_index)
                    write_sheet.write_merge(row_num, row_num, start_merge_cell, end_merge_cell - 1, this_cell_value)
                else:
                    write_sheet.write(row_num, col_index, this_cell_value)
        return write_sheet, None

    def only_write(self, input_dict_list, excel_templet_title="Sheet1"):
        """
        只写模式进行写入
        :param input_dict:[{..},{..}..]
        :param excel_templet_title: 创建sheet的名称
        :return: (data,err);
        """
        if not isinstance(input_dict_list, list):
            self.err_msg = "input_dict格式应该是[{..},{..}..]"
        try:
            if len(input_dict_list) < 1 or not isinstance(input_dict_list[0], dict):
                self.err_msg = "input_dict格式应该是[{..},{..}..]"
                return None, self.err_msg

            write_sheet = self.only_write_wb.add_sheet(excel_templet_title)
            headers = list(input_dict_list[0].keys())
            max_col = len(headers)

            # 写入头部：
            for header, col in zip(headers, range(max_col)):
                write_sheet.write(0, col, header)
            # 写入数据体
            for row in range(len(input_dict_list) - 1):
                row += 1
                this_row = input_dict_list[row]
                if not isinstance(this_row, dict):
                    continue
                for header_key, col in zip(headers, range(max_col)):
                    if not this_row.get(header_key):
                        continue
                    write_sheet.write(row, col, this_row[header_key])
            return None, None
        except Exception as e:
            err_detail = " file:" + str(e.__traceback__.tb_frame.f_globals["__file__"]) + " line:" + str(e.__traceback__.tb_lineno) if platform.python_version()[0] == "3" else ""
            return None, "系统异常：" + str(e) + err_detail

    def parse_expression(self, expression_value=None, row_input_dict=None):
        """
        把存在'{{...}}'的字符串，解析出来然后根据键值对匹配替换成 row_input_dict里面的值
        :param expression_value:
        :param row_input_dict:
        :return: 解析后的字符换
        """
        """解析表达式"""
        if not expression_value:
            return ""
        if not row_input_dict:
            return ""

        # 表达式解析数据
        this_cell_value_match = re.compile("{{.*?}}").findall(str(expression_value) if expression_value else "")
        this_cell_value_replaces = {i: row_input_dict.get(i.replace("{{", "").replace("}}", ""), "") for i in this_cell_value_match}
        for k, v in this_cell_value_replaces.items():
            expression_value = expression_value.replace(k, str(v))
        return expression_value

    def save(self, workbook=None, save_path=None, ):
        """
        保存excel，当没有save_path的时候返回流，存在则返回保存后的文件链接地址
        :param workbook: 操作的Workbook对象
        :param save_path: 保存路径，注意该路径应该是绝对路径
        :return: data,err; data: None或者文件流
        """
        # 有存在的文件路径则保存文件，否则直接返回文件流
        if not workbook or not isinstance(workbook, Workbook):
            return None, "workbook 不是一个可写入的 Workbook对象"
        if save_path:
            try:
                # 递归创建目录
                if not os.path.exists(save_path):
                    os.makedirs(save_path)
                # 保存文件
                file_path_name = (save_path + "/" + str(uuid.uuid3(uuid.NAMESPACE_DNS, str(uuid.uuid1()))) + ".xls").replace("//", "/")
                workbook.save(file_path_name)
                return file_path_name, None
            except Exception as e:
                return None, "excel 写入异常：" + str(e)
        else:
            return self.__export_stream(workbook), None

    def __export_stream(self, new_wb):
        """
        exce保存文件流
        :param new_wb: 生成新的导出工作簿对象
        :return: 文件流
        """
        try:
            from StringIO import StringIO
        except ImportError:
            from io import StringIO

        if platform.python_version()[0] == "2":
            # python 2.*.*
            output = StringIO()
            new_wb.save(output)
            output.seek(0)
            return output.getvalue()
        else:
            # python 3.*.*
            output = BytesIO()
            new_wb.save(output)
            output.seek(0)
            return output.getvalue()



# encoding: utf-8
"""
@project: djangoModel->spier_base
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 所有爬虫抽象类
@created_time: 2023/5/4 12:16
"""
from abc import ABCMeta, abstractmethod


class SpiderBase(metaclass=ABCMeta):
    key = ""  # 该爬虫类的搜索key
    url = ""  # 爬取的入口链接地址
    save_path = ""  # 保存的文件地址
    heartbeat = -1  # 爬取的频率是否需要定时暂定频率，单位为秒

    def get_links(self, *args, **kwargs):
        """
        如果是需要再二级页面获取信息则需要进行先获取链接点击
        :return:
        """
        pass

    def get_detail(self, *args, **kwargs):
        """
        获取详细信息
        :return:
        """
        pass

    @abstractmethod
    def node_select(self, *args, **kwargs):
        """
        节点选择,xpath,ID,Class，jsonpath等等
        必须继承并且重写该方法
        :return:
        """
        pass

    @abstractmethod
    def run(self, *args, **kwargs):
        """
        执行爬虫的方法，其他自定义获取
        必须继承并且重写该方法
        :return:
        """
        pass

    @abstractmethod
    def save(self, *args, **kwargs):
        """
        将爬取的文件保存到excel或者csv文件里面
        统一使用数据库迁移迁移导入数据库
        :return:
        """
        pass



# encoding: utf-8
"""
@project: djangoModel->doc_export
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis:
@created_time: 2022/11/23 16:53
"""
from io import BytesIO
import os
import platform

from docxtpl import DocxTemplate

from main.settings import BASE_DIR
from ..utils.custom_tool import write_to_log

TEMPLET_PATH_MAP = {
    "bid-online-contract": str(BASE_DIR) + "/resource_files/templet/bxtx_contract_templet.docx"  # 镖行天下合同
}


class DocExportService:
    @staticmethod
    def export(variable_params, templet_path=None, save_path=None):
        if not templet_path or not os.path.exists(templet_path):
            return None, "模板地址不正确"
        write_to_log(
            prefix="下载合同内容",
            content=variable_params
        )
        # 模板变量替换
        doc = DocxTemplate(templet_path)
        doc.render(variable_params)
        # 保存文件位置
        if not save_path is None and os.path.exists(templet_path):
            doc.save(save_path)
            return save_path, None
        # 文件路径
        return DocExportService.__export_stream(doc), None

    #  exce保存文件流
    @staticmethod
    def __export_stream(save_obj):
        try:
            from StringIO import StringIO
        except ImportError:
            from io import StringIO

        if platform.python_version()[0] == "2":
            # python 2.*.*
            output = StringIO()
            save_obj.save(output)
            output.seek(0)
            return output.getvalue()
        else:
            # python 3.*.*
            output = BytesIO()
            save_obj.save(output)
            output.seek(0)
            return output.getvalue()



import datetime
import time
import re
import json
from datetime import datetime
import pymysql
import xlrd
from main.settings import BASE_DIR
from xlrd import xldate_as_tuple
from ..services.DataConsolidation import DataConsolidation
from ..models import *


class DataExeclService:
    @staticmethod
    def excl_import(configure, table_name, file_path):
        readboot = xlrd.open_workbook(file_path)
        sheet = readboot.sheet_by_index(0)
        # # 获取excel的行和列
        nrows = sheet.nrows  # 行
        ncols = sheet.ncols  # 列
        first_row_values = sheet.row_values(0)  # 第一行数据
        list = []
        num = 1
        for row_num in range(1, nrows):
            row_values = sheet.row_values(row_num)
            if row_values:
                str_obj = {}
            for i in range(len(first_row_values)):
                ctype = sheet.cell(num, i).ctype
                cell = sheet.cell_value(num, i)
                if ctype == 2 and cell % 1 == 0.0:  # ctype为2且为浮点
                    cell = int(cell)  # 浮点转成整型
                    cell = str(cell)  # 转成整型后再转成字符串，如果想要整型就去掉该行
                elif ctype == 3:
                    date = datetime(*xldate_as_tuple(cell, 0))
                    cell = date.strftime('%Y/%m/%d %H:%M:%S')
                elif ctype == 4:
                    cell = True if cell == 1 else False
                str_obj[first_row_values[i]] = cell
            list.append(str_obj)
            num = num + 1
        configure = json.loads(configure)  # 连接数据库配置
        # 获得表字段
        field, err_txt = DataConsolidation.list_col(configure['localhost'], configure['port'], configure['username'],
                                                    configure['password'], configure['database'], table_name)
        if err_txt:
            return None, "连接数据库表失败"
        data = {
            "list": list,
            "rows": nrows - 1,
            "table": table_name,
            "field": field
        }
        return data, None

    @staticmethod
    def data_migrate(file_path, configure, export_field, old_table_id, new_table_id):
        configure = json.loads(configure)  # 连接数据库配置
        try:
            target_db = pymysql.connect(
                host=configure['localhost'],
                port=int(configure['port']),
                user=configure['username'],
                password=configure['password'],
                db=configure['database'],
                charset="utf8",
            )
        except Exception as err:
            return None, "目标数据库连接失败"
        conn = target_db.cursor()
        where = "id = " + new_table_id
        table_name_sql = "SELECT `table_name` FROM migrate_platform_table WHERE {};".format(where)
        conn.execute(table_name_sql)
        table_name = conn.fetchone()
        data = DataExeclService.excl_import(json.dumps(configure), table_name[0], file_path)
        print(data)
        target_db.commit()
        conn.close()
        data = {
            # "rows": num
        }
        return data, None



# encoding: utf-8
"""
@project: djangoModel->DataConsolidation
@author: 高栋天
@Email: 1499593644@qq.com
@synopsis: 数据迁移相关操作
@created_time: 2023/4/27 9:57
"""
import json

import pymysql


class DataConsolidation:
    def __init__(self):
        pass

    @staticmethod
    def consolidation(configure, table_name, configure_configure, target_table_name, export_field, write_field):
        """
        将一张表的指定字段数据导出到另一张表
        @param table_name 表名
        @param export_field 表字段
        @param target_table_name 目标表名
        @param write_field 目标字段

        """
        # 边界检查
        export_field_list = export_field.split(",")
        write_field_list = write_field.split(",")

        if not table_name:
            # print(u"错误，表名必填")
            return None, "错误，表名必填"
        if not target_table_name:
            # print(u"错误，表名必填")
            return None, "错误，目标表名必填"
        if len(export_field_list) != len(write_field_list):
            return None, "错误，替换字段长度不符"

        configure = json.loads(configure)  # 连接数据库配置
        configure_configure = json.loads(configure_configure)  # 连接目标数据库配置
        # 连接数据库
        try:
            db = pymysql.connect(
                host=configure['localhost'],
                port=int(configure['port']),
                user=configure['username'],
                password=configure['password'],
                db=configure['database'],
                charset="utf8",
            )
        except Exception as err:
            return None, "数据库连接失败"
        cursor = db.cursor()
        # 连接目标数据库
        try:
            target_db = pymysql.connect(
                host=configure_configure['localhost'],
                port=int(configure_configure['port']),
                user=configure_configure['username'],
                password=configure_configure['password'],
                db=configure_configure['database'],
                charset="utf8",
            )
        except Exception as err:
            return None, "目标数据库连接失败"
        conn = target_db.cursor()
        # cursor = connection.cursor()
        # 查询要导出的数据
        query_sql = "select {} from {}".format(export_field, table_name)
        cursor.execute(query_sql)
        # cols = [desc[0] for desc in cursor.description]
        rows = cursor.fetchall()
        try:
            for row in rows:
                sql = "INSERT INTO `{}` ({}) VALUES {};".format(target_table_name, write_field, row)
                sql = sql.replace("'None'", "NULL").replace("None", "NULL")
                conn.execute(sql)
        except Exception as e:
            return None, e

        db.commit()
        target_db.commit()
        cursor.close()
        conn.close()
        return None, None

    @staticmethod
    def list_col(localhost, port, username, password, database, table_name):
        """
        查询所有字段
        @param localhost 连接地址
        @param username 用户名
        @param password 连接密码
        @param database 数据库名
        @param table_name 表名
        """
        try:
            db = pymysql.connect(
                host=localhost,
                port=int(port),
                user=username,
                password=password,
                db=database,
                charset="utf8",
            )
        except Exception as err:
            return None, "数据库连接失败"

        cursor = db.cursor()
        cursor.execute("select * from %s" % table_name)
        col_name_list = [tuple[0] for tuple in cursor.description]
        db.close()
        return col_name_list, None

    # 列出所有的表
    @staticmethod
    def list_table(localhost, port, username, password, database):
        """
         列出所有的表
         @param localhost 连接地址
         @param username 用户名
         @param password 连接密码
         @param database 数据库名

        """
        try:
            db = pymysql.connect(
                host=localhost,
                port=int(port),
                user=username,
                password=password,
                db=database,
                charset="utf8",
            )
        except Exception as err:
            return None, "数据库连接失败"

        cursor = db.cursor()
        cursor.execute("show tables")
        table_list = [tuple[0] for tuple in cursor.fetchall()]
        db.close()
        return table_list, None

# tables = list_table(localhost, username, password, database) # 获取所有表，返回的是一个可迭代对象
# print(tables)
#
# for table in tables:
#     col_names = list_col(localhost, username, password, database, table)
#     print(col_names) # 输出所有字段名



from django.db import models
from xj_user.models import Platform


# Create your models here.

# 1、migrate_platform_table  迁移平台表 [NF1]
class MigratePlatformTable(models.Model):
    plaform = models.ForeignKey(Platform, verbose_name='用户', on_delete=models.DO_NOTHING, db_column='plaform_id',
                                help_text='')
    # plaform_id = models.IntegerField(verbose_name='平台id', blank=True, null=True, db_index=True)
    table_name = models.CharField(verbose_name='平台表名', max_length=128, blank=True, null=True, db_index=True)

    class Meta:
        db_table = 'migrate_platform_table'
        verbose_name_plural = "01. 迁移平台表"

    def __str__(self):
        return f"{self.plaform_id}"


# 2、migrate_old_to_new  迁移旧表到新表id的映射表 [NF1]
class MigrateOldToNew(models.Model):
    old_table = models.ForeignKey(MigratePlatformTable, verbose_name='旧表id', on_delete=models.DO_NOTHING,
                                  related_name="old_table")
    new_table = models.ForeignKey(MigratePlatformTable, verbose_name='旧表id', on_delete=models.DO_NOTHING,
                                  related_name="new_table"
                                  )
    old_data_id = models.IntegerField(verbose_name='旧表主键id', blank=True, null=True, db_index=True)
    new_data_id = models.IntegerField(verbose_name='新表主键id', blank=True, null=True, db_index=True)
    create_time = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, )
    modify_time = models.DateTimeField(verbose_name='修改时间', auto_now=True, blank=True, null=True, )

    class Meta:
        db_table = 'migrate_old_to_new'
        verbose_name_plural = "02. 迁移旧表到新表id的映射表"

    def __str__(self):
        return f"{self.old_table_id}"


# 3、migrate_table_key_map  迁移键名映射表 [NF1]
class MigrateTableKeyMap(models.Model):
    old_table = models.ForeignKey(MigratePlatformTable, verbose_name='旧表id', on_delete=models.DO_NOTHING,
                                  related_name="old_tables")
    new_table = models.ForeignKey(MigratePlatformTable, verbose_name='旧表id', on_delete=models.DO_NOTHING,
                                  related_name="new_tables"
                                  )
    old_key_name = models.CharField(verbose_name='旧表映射字段', max_length=255, blank=True, null=True, help_text='')
    new_key_name = models.CharField(verbose_name='新表映射字段', max_length=255, blank=True, null=True, help_text='')
    is_primary_key = models.IntegerField(verbose_name='是否主键idid', blank=True, null=True,)
    config = models.CharField(verbose_name='配置', max_length=255, blank=True, null=True, help_text='')

    class Meta:
        db_table = 'migrate_table_key_map'
        verbose_name_plural = "03. 迁移键名映射表"

    def __str__(self):
        return f"{self.old_table_id}"



# encoding: utf-8
"""
@project: PythonSuperMario-master->config
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 配置文件
@created_time: 2022/11/13 11:08
"""

import configparser
from configparser import ConfigParser
import os

from main.settings import BASE_DIR


class JConfig(ConfigParser):
    def __init__(self, *args, **kwargs):
        ConfigParser.__init__(self, *args, **kwargs)
        self.read()

    def read(self, filenames=BASE_DIR, encoding="utf-8-sig"):
        """获取配置文件，内容"""
        super().read(filenames, encoding)

    @staticmethod
    def get_section(path, section, encoding="utf-8-sig"):
        if not os.path.exists(path):
            return {}

        config = configparser.ConfigParser()
        config.read(path, encoding=encoding)
        if not config.has_section(section):
            return {}
        tuple_list = config.items(section)
        return {k: v for k, v in tuple_list}



from rest_framework.views import APIView

# from ..services.data_execl_service2 import DataExeclService
from ..services.data_execl_service import DataExeclService
from ..utils.custom_response import util_response


class DataExcel(APIView):
    def data_proving(self):
        """
        数据迁移
        @param configure 连接配置（json）
        @param table_name 表名
        """
        configure = self.POST.get('configure')
        filename = self.POST.get('filename')  # 文件路径
        table_name = self.POST.get('table_name')
        data, err_txt = DataExeclService.excl_import(configure, table_name, filename)
        if not err_txt:
            return util_response(data=data)
        return util_response(err=47767, msg=err_txt)

    def data_match_write(self):
        filename = self.POST.get('filename')  # 文件路径
        configure = self.POST.get('configure')
        export_field = self.POST.get('export_field')
        old_table_id = self.POST.get('old_table_id')
        new_table_id = self.POST.get('new_table_id')
        data, err_txt = DataExeclService.data_migrate(filename, configure, export_field, old_table_id,
                                                      new_table_id)
        if not err_txt:
            return util_response(data=data)
        return util_response(err=47767, msg=err_txt)

    def data_cover(self):
        filename = self.POST.get('filename')  # 文件路径
        table_name = self.POST.get('table_name')
        configure = self.POST.get('configure')
        cover_where = self.POST.get('cover_where')
        cover_field = self.POST.get('cover_field')
        data, err_txt = DataExeclService.data_cover(filename, configure, table_name, cover_where, cover_field)
        if not err_txt:
            return util_response(data=data)
        return util_response(err=47767, msg=err_txt)

    """
    def data_cover2(self):
        filename = self.POST.get('filename')  # 文件路径
        table_name = self.POST.get('table_name')
        configure = self.POST.get('configure')
        cover_where = self.POST.get('cover_where')
        cover_field = self.POST.get('cover_field')
        data, err_txt = DataExeclService.data_cover2(filename, configure, table_name, cover_where, cover_field)
        if not err_txt:
            return util_response(data=data)
        return util_response(err=47767, msg=err_txt)"""



import json

import pymysql


class MappingProcess:
    def __init__(self):
        pass

    @staticmethod
    def handle(configure):
        configure = json.loads(configure)  # 连接数据库配置
        try:
            db = pymysql.connect(
                host=configure['localhost'],
                port=int(configure['port']),
                user=configure['username'],
                password=configure['password'],
                db=configure['database'],
                charset="utf8",
                cursorclass=pymysql.cursors.DictCursor
            )
        except Exception as err:
            return None, "目标数据库连接失败"
        conn = db.cursor()

        try:
            # 查询表字段映射
            foreign_key_mapping_sql = "SELECT a.*,b.table_name as table_name,c.table_name as map_table_name FROM migrate_table_key_map a LEFT JOIN migrate_platform_table b ON a.old_table_id = b.id LEFT JOIN migrate_platform_table c ON a.new_table_id= c.id "
            conn.execute(foreign_key_mapping_sql)
            data = conn.fetchall()

            for v in data:
                # 如果是主键则更改关联外键否则去根据配置更改
                if v['is_primary_key'] == 1:
                    # 查询映射表中映射关系    旧表id->新表id
                    map_sql = "SELECT * FROM migrate_old_to_new WHERE is_handle = 0 and  new_table_id = {};".format(
                        v['new_table_id'])
                    conn.execute(map_sql)
                    map_data = conn.fetchall()
                    sql_list = []
                    for m in map_data:
                        sql = "SELECT * FROM {} WHERE {} = {};".format(v['table_name'], v['old_key_name'],
                                                                       m['old_data_id'])
                        conn.execute(sql)
                        data = conn.fetchone()
                        if isinstance(data, dict):
                            # "UPDATE 表名 SET 映射字段id = 导入新产生的映射id WHERE 映射字段id = 老数据id AND id = （主键id）"
                            update_sql = "UPDATE {} SET {} = {} WHERE {} = {} AND id = {};".format(v['table_name'],
                                                                                                   v['old_key_name'],
                                                                                                   m['new_data_id'],
                                                                                                   v['old_key_name'],
                                                                                                   m['old_data_id'],
                                                                                                   data['id'])
                            print(update_sql)

                            # 为了不重复更改已修改的映射数据 改过的数据进行标注
                            sql = "UPDATE migrate_old_to_new SET  is_handle = 1 WHERE id = {} ;".format(m['id'])
                            sql_list.append(update_sql)
                            sql_list.append(sql)

                    for s in sql_list:
                        conn.execute(s)
                else:
                    config = json.loads(v['config'])
                    sql = "SELECT * FROM {};".format(v['table_name'])
                    conn.execute(sql)
                    data = conn.fetchall()
                    for d in data:
                        new_key_name = str(d[v['new_key_name']])
                        if new_key_name in config:
                            update_sql = "UPDATE {} SET {} = '{}' WHERE {} = '{}' AND id = {};".format(v['table_name'],
                                                                                                       v[
                                                                                                           'new_key_name'],
                                                                                                       config[
                                                                                                           new_key_name],
                                                                                                       v[
                                                                                                           'new_key_name'],
                                                                                                       new_key_name,
                                                                                                       d['id'])

                            conn.execute(update_sql)

            db.commit()
            conn.close()
        except Exception as err:
            return None, "脚本执行失败"

        return None, None



# -*- encoding:utf-8 -*-
import time, os
from datetime import datetime
from hashlib import md5


class DigitAlgorithm:

    @staticmethod
    # 生成交易号：2位数（当前年份后2位数字）+8位数（当前时间戳去头2位）+6位数（用户名 经过hash crc16生成的 4位十六进制 转成5位数 然后头为补0）
    def make_unicode_16(salt=''):
        # 当前时间戳
        date_time = time.localtime(time.time())
        # 截取第3位到第4位
        year_code = str(date_time.tm_year)[2:4]

        # 当前时间戳
        timestamp = str(int(time.time()))
        # 截取第3位到第10位
        timestamp_code = timestamp[2:10]

        # 十六进制校验码
        crc_hex = DigitAlgorithm.crc16(salt) if salt else '0'
        # 十六进制转十进制
        crc_int = int(crc_hex, 16)
        # 头位补0
        crc_code = str('000000' + str(crc_int))[-6:]
        unicode = year_code + timestamp_code + crc_code

        return unicode

    # 生成日期yyyymmdd（8位）+时间（6位）hhmmss+毫秒（3位）=17位日期时间型数字
    @staticmethod
    def make_datetime_17():
        # 8位数+6+3=17位数
        t = time
        ms = int((t.time() - int(t.time()))*1000)
        return t.strftime('%Y%m%d%H%M%S',) + str(ms)

    @staticmethod
    def make_datetime_14():
        return time.strftime('%Y%m%d%H%M%S', )

    # crc16
    @staticmethod
    def crc16(x):
        a = 0xFFFF
        b = 0xA001
        for byte in x:
            a ^= ord(byte)
            for i in range(8):
                last = a % 2
                a >>= 1
                if last == 1:
                    a ^= b
        s = hex(a).upper()

        return s[2:6]

    # 文件生成md5
    @staticmethod
    def make_file_md5(file):
        if not os.path.exists(file):
            return None
        f = open(file, "rb")
        m = md5()
        m.update(f.read())
        f.close()
        result = m.hexdigest()
        return result










# encoding: utf-8
"""
@project: djangoModel->hangzhou_spider
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 杭州市的爬取文件
@created_time: 2023/5/4 12:15
"""
import datetime
import logging
import os
from time import sleep

from lxml import etree
import requests
from selenium.webdriver.chrome import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.remote.remote_connection import LOGGER

from config.config import JConfig
from xj_migrate.services.excel_export_service import ExcelExport
from xj_migrate.utils.custom_tool import write_to_log
from ..spider_base import SpiderBase

LOGGER.setLevel(logging.ERROR)

main_config = JConfig()


class HanZhouSpider(SpiderBase):
    key = "HangZhou"  # 该爬虫类的搜索key
    url = "https://ggzy.hzctc.hangzhou.gov.cn/SecondPage/ProjectAfficheList?title=&area=&afficheType=22"  # 爬取的入口链接地址
    header = {
        "Host": "ggzy.hzctc.hangzhou.gov.cn",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.0.0 Safari/537.36 Edg/112.0.1722.68",
    }

    save_path = main_config.get_base_dir() + "/resource_files/spider/" + datetime.date.today().strftime('%y%m%d') + "/"  # 保存的文件地址
    save_file_name = datetime.date.today().strftime('%y%m%d') + key + ".xls"  # 保存的文件名
    heartbeat = -1  # 爬取的频率是否需要定时暂定频率，单位为秒。非强制字段属性

    def __init__(self):
        """外层调用方面请做好"""
        if not self.save_path:
            raise Exception("没有找到爬取文件的保存路径")
        # request会话对象初始化
        self.session = requests.session()
        # 浏览器初始化
        chrome_options = webdriver.Options()
        chrome_options.add_experimental_option('excludeSwitches', ['enable-logging'])  # 禁止打印日志
        chrome_options.add_experimental_option('excludeSwitches', ['enable-automation'])  # 实现了规避监测
        self.browser = webdriver.WebDriver(options=chrome_options)
        self.browser.maximize_window()
        self.browser.implicitly_wait(10)

    def get_links(self):
        """
        如果是需要再二级页面获取信息则需要进行先获取链接点击
        :return:
        """
        self.browser.get(self.url)
        sleep(3)
        links = []
        try:
            for i in self.browser.find_elements(by=By.XPATH, value="//tbody/tr/td/a"):
                links.append(i.get_attribute("href").replace("undefined", "17"))
            self.browser.close()
        except Exception as e:
            print("错误捕捉：", str(e))
        return links

    def get_detail(self, link):
        response = self.session.get(url=link, headers=self.header)
        return response.text

    def node_select(self, response_html=None):
        """
        节点选择,xpath,ID,Class等等
        必须继承并且重写该方法
        :return:
        """
        if not response_html:
            return {}
        node_dict = {}
        # 网页写入本地
        # with open("D:\\tempMaterial\\desktop\download.html", "w", encoding="utf-8") as fp:
        #     fp.write(response_html)
        info_html = etree.HTML(response_html)
        title_xpath = "//div[@class='MainList']//div[@class='WordSection1']/div[1]//tr[1]/td[2]/p[@class='MsoNormal']//span/text()"
        title = info_html.xpath(title_xpath)
        node_dict["title"] = title[0] if title and isinstance(title, list) else title
        if not node_dict["title"]:
            return None
        return node_dict

    def run(self):
        """
        执行爬虫的方法，其他自定义获取
        必须继承并且重写该方法
        :return:
        """
        links = self.get_links()
        save_dict_list = []
        for link in links[1:]:
            html_str = self.get_detail(link)
            detail_info = self.node_select(html_str)
            if not detail_info:
                continue
            save_dict_list.append(detail_info)
        print("save_dict_list:", save_dict_list)
        return self.save(save_dict_list=save_dict_list)

    def save(self, save_dict_list=None):
        """
        将爬取的文件保存到excel或者csv文件里面
        统一使用数据库迁移迁移导入数据库
        :return:
        """
        if not os.path.exists(self.save_path):
            os.makedirs(self.save_path)
        if not save_dict_list:
            return

        try:
            export_instance = ExcelExport()
            data, err = export_instance.only_write(input_dict_list=save_dict_list)
            return export_instance.save(workbook=export_instance.only_write_wb, save_path=self.save_path)
        except Exception as e:
            write_to_log(err_obj=e)



# _*_coding:utf-8_*_

import logging

from rest_framework import generics
from rest_framework.response import Response

from ..services.migrate_foregin_key_mapping_service import MigrateForeignKeyMappingService
from ..utils.model_handle import parse_data, util_response

logger = logging.getLogger(__name__)


# 迁移键名映射
class MigrateForeignKeyMapping(generics.UpdateAPIView):  # 或继承(APIView)

    def get(self, request, *args, **kwargs):
        data, err_txt = MigrateForeignKeyMappingService.get()
        if err_txt is None:
            return util_response(data=data)
        return util_response(err=47767, msg=err_txt)

    def post(self, request, *args, **kwargs):
        params = parse_data(request)
        if not params:
            return util_response(err=6046, msg='至少需要一个请求参数')
        data, err_txt = MigrateForeignKeyMappingService.post(params=params)
        if err_txt is None:
            return util_response(data=data)
        return util_response(err=47767, msg=err_txt)



# encoding: utf-8
"""
@project: djangoModel->export_api
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 导出AOIView
@created_time: 2022/11/23 18:03
"""
import datetime

from rest_framework.views import APIView

from main.settings import MEDIA_ROOT
from xj_user.services.user_detail_info_service import DetailInfoService
from ..services.doc_export_service import DocExportService, TEMPLET_PATH_MAP
from ..services.excel_export_service import ExcelExport
from ..utils.custom_response import doc_response, excel_response, util_response
from ..utils.custom_tool import request_params_wrapper
from ..utils.user_wrapper import user_authentication_wrapper


class exportAPIView(APIView):

    @user_authentication_wrapper
    @request_params_wrapper
    def doc_export(self, *args, request_params=None, user_info=None, **kwargs):
        if request_params is None:
            request_params = {}
        if user_info is None:
            user_info = {}

        templet_path_name = request_params.pop("templet_path_name", None)
        if templet_path_name is None or TEMPLET_PATH_MAP.get(templet_path_name, None) is None:
            return util_response(err=1000, msg="找不到导出模板")

        user_id = request_params.get("user_id", None) or user_info.get("user_id", None)
        try:
            user_id = int(user_id)
        except ValueError:
            user_id = None
        if user_id is None:
            return util_response(err=1001, msg="找不到用户信息")

        user_detail_info, detail_err = DetailInfoService.get_detail(user_id=user_id)
        if not detail_err is None:
            return util_response(err=1002, msg=detail_err)
        request_params.update(user_detail_info)
        today = datetime.date.today()
        request_params.update({"current_date": today.strftime('%Y-%m-%d')})
        data, export_err = DocExportService.export(request_params, templet_path=TEMPLET_PATH_MAP.get(templet_path_name))
        if not export_err is None:
            return util_response(err=1003, msg=export_err)
        return doc_response(data)

    @request_params_wrapper
    def excel_export(self, *args, request_params, **kwargs):
        templet_path = MEDIA_ROOT + "/templet/test.xls"
        export_instance = ExcelExport(excel_templet_path=templet_path)
        # 写入
        data, err = export_instance.additional_write(input_dict=[request_params])
        # 保存
        excel_data, err = export_instance.save(workbook=export_instance.additional_write_wb)
        return excel_response(excel_data)



from rest_framework.views import APIView

from ..services.DataConsolidation import DataConsolidation
from ..services.mapping_process import MappingProcess
from ..utils.custom_response import util_response


class DataMigration(APIView):
    def consolidation(self):
        """
        数据迁移
        @param configure 连接配置（json）
        @param table_name 表名
        @param export_field 查询字段（逗号分割）
        @param configure 目标连接配置（json）
        @param target_table_name 目标表名
        @param write_field 目标字段（逗号分割）

        """
        configure = self.POST.get('configure')
        table_name = self.POST.get('table_name')
        export_field = self.POST.get('export_field')
        target_configure = self.POST.get('target_configure')
        target_table_name = self.POST.get('target_table_name', "")
        write_field = self.POST.get('write_field', "")

        data, err_txt = DataConsolidation.consolidation(configure, table_name, target_configure, target_table_name,
                                                        export_field, write_field)
        if not err_txt:
            return util_response(data=data)
        return util_response(err=47767, msg=err_txt)

    def list_table(self):
        """
        查询所有字段
        @param localhost 连接地址
        @param username 用户名
        @param password 连接密码
        @param database 数据库名
        @param tabls_name 表名

        """
        localhost = self.POST.get('localhost', '127.0.0.1')
        port = self.POST.get('port', 3306)
        username = self.POST.get('username', "root")
        password = self.POST.get('password', "")
        database = self.POST.get('database', "")

        data, err_txt = DataConsolidation.list_table(localhost, port, username, password, database)
        if not err_txt:
            return util_response(data=data)
        return util_response(err=47767, msg=err_txt)

    def list_col(self):
        """
        查询所有字段
        @param localhost 连接地址 192.168.2.252
        @param username 用户名 admin
        @param password 连接密码 Hh.v0254
        @param database 数据库名 spss_muztak_cn
        @param table_name 表名

        """
        localhost = self.POST.get('localhost', '127.0.0.1')
        port = self.POST.get('port', 3306)
        username = self.POST.get('username', "root")
        password = self.POST.get('password', "")
        database = self.POST.get('database', "")
        table_name = self.POST.get('table_name', "")

        data, err_txt = DataConsolidation.list_col(localhost, port, username, password, database, table_name)
        if not err_txt:
            return util_response(data=data)
        return util_response(err=47767, msg=err_txt)

    def mapping_rocessing(self):
        configure = self.POST.get('configure')
        # data, err_txt = MappingProcess.handle(configure).delay()
        data, err_txt = MappingProcess.handle(configure)
        if not err_txt:
            return util_response(data=data)
        return util_response(err=47767, msg=err_txt)



# 应用名称
from django.urls import re_path

from .apis import data_migration, data_excel, migrate_platfrom, migrate_foreign_key_mapping, export_api

app_name = 'database'

urlpatterns = [

    re_path(r'^list_table/?$', data_migration.DataMigration.list_table),
    re_path(r'^list_col/?$', data_migration.DataMigration.list_col),
    re_path(r'^consolidation/?$', data_migration.DataMigration.consolidation),
    re_path(r'^data_proving/?$', data_excel.DataExcel.data_proving),
    re_path(r'^data_match_write/?$', data_excel.DataExcel.data_match_write),
    re_path(r'^data_cover/?$', data_excel.DataExcel.data_cover),
    re_path(r'^mapping_rocessing/?$', data_migration.DataMigration.mapping_rocessing),
    re_path(r'^platform/?$', migrate_platfrom.MigratePlatfrom.as_view()),  # 迁移平台
    re_path(r'^foreign_key_mapping/?$', migrate_foreign_key_mapping.MigrateForeignKeyMapping.as_view()),  # 外键映射
    re_path(r'^doc_export/?$', export_api.exportAPIView.doc_export),  # 外键映射
    re_path(r'^excel_export/?$', export_api.exportAPIView.excel_export),  # 外键映射

]



# encoding: utf-8
"""
@project: djangoModel->migrate_by_spider
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 镖行业务流程爬取数据
@created_time: 2023/5/4 11:19
"""

from django.core.management.base import BaseCommand

from xj_migrate.management.bx_bids_spiders.hangzhou_spider import HanZhouSpider
from xj_migrate.utils.custom_tool import write_to_log


# note 待完成脚本的爬取的网站
# "NingBo": "http://ggzy.zwb.ningbo.gov.cn/cms/jyxxgcjs/index.htm",
# "JiaXing": "http://hn.jxzbtb.cn/jygg/003001/subpagesecond.html",
# "HuiZhou": "http://ggzyjy.huzhou.gov.cn/",
# "ShaoXing": "http://ol.ggb.sx.gov.cn/jsgc/001001/sec.html",
# "JinHua": "http://ggzyjy.jinhua.gov.cn/col/col1229641554/index.html",
# "XuZhou": "https://ggzy.qz.gov.cn/col/col1229683626/index.html?number=ggjyA004",
# "ZhouShan": "http://zsztb.zhoushan.gov.cn/col/col1229679798/index.html?key",
# "TaiZhou": "https://tzztb.zjtz.gov.cn/tzcms/gcjy/index.htm",
# "LiShui": "https://lssggzy.lishui.gov.cn/"


class Command(BaseCommand):
    # 帮助文本, 一般备注命令的用途及如何使用。
    help = "爬取招投标咨询,并保存到文件中"

    spider_city_instance = {
        "HangZhou": HanZhouSpider
    }

    # 给命令添加一个名为name的参数
    def add_arguments(self, parser):
        pass

    # 核心业务逻辑，通过options字典接收name参数值，拼接字符串后输出
    def handle(self, *args, **options):
        for key, instance in self.spider_city_instance.items():
            try:
                instance().run()
            except Exception as e:
                write_to_log(prefix="打印日志系統叫脚本", err_obj=e)



# encoding: utf-8
"""
@project: djangoModel->setup
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 模块打包文件
@created_time: 2022/6/18 15:14
"""
from setuptools import setup, find_packages

with open('README.md', 'r', encoding='utf8') as fp:
    log_desc = fp.read()

setup(
    name='xj-migrate',  # 模块名称
    version='1.0.16',  # 模块版本
    description='迁移模块',  # 项目 摘要描述
    long_description=log_desc,  # 项目描述
    long_description_content_type="text/markdown",  # md文件，markdown格式
    packages=find_packages(),  # 系统自动从当前目录开始找包
    # packages=['xj_migrate'],  # 系指定安装模块
    license="apache 3.0",
    install_requires=[
        "xj_user",
        "pymysql",
        "xlrd",
        "xlutils",
        "xlwt",
        "docxtpl"
    ]
)
"""
name : 打包后包的文件名
version : 版本号
author : 作者
author_email : 作者的邮箱
py_modules : 要打包的.py文件
packages: 打包的python文件夹
include_package_data : 项目里会有一些非py文件,比如html和js等,这时候就要靠include_package_data 和 package_data 来指定了。package_data:一般写成{‘your_package_name’: [“files”]}, include_package_data还没完,还需要修改MANIFEST.in文件.MANIFEST.in文件的语法为: include xxx/xxx/xxx/.ini/(所有以.ini结尾的文件,也可以直接指定文件名)
license : 支持的开源协议
description : 对项目简短的一个形容
ext_modules : 是一个包含Extension实例的列表,Extension的定义也有一些参数。
ext_package : 定义extension的相对路径
requires : 定义依赖哪些模块
provides : 定义可以为哪些模块提供依赖
data_files :指定其他的一些文件(如配置文件),规定了哪些文件被安装到哪些目录中。如果目录名是相对路径,则是相对于sys.prefix或sys.exec_prefix的路径。如果没有提供模板,会被添加到MANIFEST文件中。
"""



from django.db.models import F

from ..models import MigrateTableKeyMap


class MigrateForeignKeyMappingService:

    @staticmethod
    def get():
        currencies = MigrateTableKeyMap.objects.all().annotate(old_table_name=F('old_table__table_name'),
                                                               new_table_name=F('new_table__table_name'))

        return list(
            currencies.values('old_table_name', 'new_table_name', 'old_table_id', 'new_table_id', 'old_key_name',
                              'new_key_name')), None

    @staticmethod
    def post(params):
        old_table_id = params.get('old_table_id', '')
        new_table_id = params.get('new_table_id', '')
        old_key_name = params.get('old_key_name', '')
        new_key_name = params.get('new_key_name', '')
        config = params.get('config', '')
        pay_mode_set = MigrateTableKeyMap.objects.filter(old_table_id=old_table_id, new_table_id=new_table_id,
                                                         old_key_name=old_key_name, new_key_name=new_key_name).first()
        if pay_mode_set is not None:
            return None, "数据已存在"
        try:
            MigrateTableKeyMap.objects.create(**params)
            return None, None
        except Exception as e:
            return None, "参数配置错误：" + str(e)



import os
import base64
import hashlib
from datetime import datetime
from urllib import parse
from pathlib import Path
from main.settings import BASE_DIR
from .j_config import JConfig
from .j_dict import JDict

import OpenSSL
import requests

module_root = str(Path(__file__).resolve().parent)
# 配置之对象
main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))
module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))
# 密钥地址
cert_path = main_config_dict.UNIONPAY_CERT_PATH or module_config_dict.UNIONPAY_CERT_PATH or (
        str(BASE_DIR) + "/config/acp_test_sign.pfx")

cert_password = main_config_dict.unionpay_cert_password
# print(main_config_dict.unionpay_cert_password)


class UnionPay:
    """
    银联支付接口（PC端）
    """

    def __init__(
            self,
            version,
            mer_id,
            front_url,
            back_url,
            backend_url,
            cert_path,
            debug=False
    ):
        self.version = version
        self.mer_id = mer_id
        self.front_url = front_url
        self.back_url = back_url
        self.backend_url = backend_url
        self.cert = {}
        self.cert_id = self.__get_cert_id(cert_path)

        if debug is True:
            # 支付网关
            self.gateway = "https://gateway.test.95516.com/gateway/api/frontTransReq.do"
            # 查询网关
            self.query_gateway = "https://gateway.test.95516.com/gateway/api/queryTrans.do"
        else:
            self.gateway = "https://gateway.95516.com/gateway/api/frontTransReq.do"
            self.query_gateway = "https://gateway.95516.com/gateway/api/queryTrans.do"

    def build_request_data(self, order_id, txn_amt, **kwargs):
        """
        构建请求数据
        :param order_id: 商户订单号
        :param txn_amt: 交易金额(单位: 分)
        :return:
        """
        request_data = {
            "version": self.version,  # 版本
            "encoding": "utf-8",  # 编码
            "txnType": "01",  # 交易类型  01：消费
            "txnSubType": "01",  # 交易子类  01：自助消费
            "bizType": "000201",  # 产品类型  000201：B2C网关支付
            "frontUrl": self.front_url,  # 前台通知地址
            "backUrl": self.back_url,  # 后台通知地址 需外网
            "signMethod": "01",  # 签名方法  01：RSA签名
            "channelType": "07",  # 渠道类型  07：互联网
            "accessType": "0",  # 接入类型  0：普通商户直连接入
            "currencyCode": "156",  # 交易币种  156：人民币
            "merId": self.mer_id,  # 商户代码
            "txnAmt": txn_amt,  # 订单金额(单位: 分)
            "txnTime": datetime.now().strftime("%Y%m%d%H%M%S"),  # 订单发送时间
            "certId": self.cert_id,
            "orderId": order_id,
            "signature": ""
        }
        request_data.update(**kwargs)
        self.get_sign(request_data)
        return request_data

    def pay_url(self, request_data):
        payment_url = "{}?{}".format(self.backend_url, parse.urlencode(request_data))
        return payment_url

    def pay_html(self, request_data):
        result = """<html>
             <head>
                 <meta http-equiv="Content-Type" content="text/html"charset="utf-8"/>
             </head>
             <body onload="pay_form.submit();">
                 <form id="pay_form" name="pay_form" action="{}" method="post">""".format(self.gateway)
        for key, value in request_data.items():
            result += """<input type="hidden" name="{0}" id="{0}" value="{1}"/>""".format(key, value)
        result = result + """<!-- <input type="submit" type="hidden">--></form></body></html>"""
        return result

    def get_sign(self, data):
        """
        获取签名
        :param data:
        :return:
        """
        sha256 = hashlib.sha256(self.build_sign_str(data).encode("utf-8")).hexdigest()
        private = OpenSSL.crypto.sign(self.cert["pkey"], sha256, "sha256")
        data["signature"] = str(base64.b64encode(private), encoding="utf-8")

    def verify_sign(self, data):
        """
        银联回调签名校验
        """
        signature = data.pop('signature')  # 获取签名
        link_string = self.build_sign_str(data)
        digest = hashlib.sha256(bytes(link_string, encoding="utf-8")).hexdigest()
        signature = base64.b64decode(signature)
        sign_pubkey_cert = data.get("signPubKeyCert", None)

        try:
            x509_ert = OpenSSL.crypto.load_certificate(OpenSSL.crypto.FILETYPE_PEM, sign_pubkey_cert)
            OpenSSL.crypto.verify(x509_ert, signature, digest, 'sha256')
            return True
        except Exception as exc:
            return False

    def verify_query(self, order_num, txn_time):
        '''
        验证查询的交易状态
        :param order_num: 订单号
        :param txn_time: 交易时间
        :return: True or False
        '''
        request_data = {
            "version": self.version,
            "encoding": "utf-8",
            "txnType": "00",  # 00:查询
            "txnSubType": "00",
            "bizType": "000201",
            "signMethod": "01",  # 签名方法  01：RSA签名
            "accessType": "0",
            "merId": self.mer_id,
            "txnTime": txn_time,
            "orderId": order_num,
            "certId": self.cert_id,
        }

        self.get_sign(request_data)
        request_data['signature'] = parse.urlencode({'signature': request_data['signature']})[10:]
        req_string = self.build_sign_str(request_data)

        res = requests.post(
            url=self.query_gateway,
            data=req_string,
            headers={
                'content-type': 'application/x-www-form-urlencoded'
            }
        )
        if res.status_code != requests.codes.ok:
            query_status = False
        else:
            content = self.parse_arguments(res.content.decode('utf-8'))
            if content.get('origRespCode', '') == '00':
                query_status = True
            else:
                query_status = False
        return query_status

    def __get_cert_id(self, cert_path):
        """
        获取证书ID(签名KEY)
        :param cert_path:
        :return:
        """
        with open(cert_path, "rb") as f:
            print('<啊啊啊啊>', cert_path)
            certs = OpenSSL.crypto.load_pkcs12(f.read(), cert_password)
            x509data = certs.get_certificate()
            self.cert["certid"] = x509data.get_serial_number()
            self.cert["pkey"] = certs.get_privatekey()

        return self.cert["certid"]

    @staticmethod
    def build_sign_str(data):
        """
        排序
        :param data:
        :return:
        """
        req = []
        for key in sorted(data.keys()):
            if data[key] != '':
                req.append("%s=%s" % (key, data[key]))

        return '&'.join(req)

    @staticmethod
    def parse_arguments(raw):
        """
        :param raw: raw data to parse argument
        :return:
        """
        data = {}
        qs_params = parse.parse_qs(str(raw))
        for name in qs_params.keys():
            data[name] = qs_params.get(name)[-1]
        return data



import datetime
import re
from datetime import datetime
import json
import time
from decimal import Decimal
import math
import random

import pymysql
import pytz
import xlrd
from django.utils import timezone
from xlrd import xldate_as_tuple

from ..services.DataConsolidation import DataConsolidation


class DataExeclService:
    @staticmethod
    def excl_import(configure, table_name, file_path):
        # save_dir = "static/upload"
        # filename = "user_base_info.xlsx"
        # 文件地址
        # file_path = re.sub(r"[/\\]{1,3}", "/", f"{str(BASE_DIR)}/{save_dir}/{filename}")
        # 打开上传 excel 表格
        readboot = xlrd.open_workbook(file_path)
        sheet = readboot.sheet_by_index(0)
        # # 获取excel的行和列
        nrows = sheet.nrows  # 行
        ncols = sheet.ncols  # 列
        first_row_values = sheet.row_values(0)  # 第一行数据
        list = []
        num = 1
        for row_num in range(1, nrows):
            row_values = sheet.row_values(row_num)
            if row_values:
                str_obj = {}
            for i in range(len(first_row_values)):
                ctype = sheet.cell(num, i).ctype
                cell = sheet.cell_value(num, i)
                if ctype == 2 and cell % 1 == 0.0:  # ctype为2且为浮点
                    cell = int(cell)  # 浮点转成整型
                    cell = str(cell)  # 转成整型后再转成字符串，如果想要整型就去掉该行
                elif ctype == 3:
                    date = datetime(*xldate_as_tuple(cell, 0))
                    cell = date.strftime('%Y/%m/%d %H:%M:%S')
                elif ctype == 4:
                    cell = True if cell == 1 else False
                str_obj[first_row_values[i]] = cell
            list.append(str_obj)
            num = num + 1
        configure = json.loads(configure)  # 连接数据库配置
        # 获得表字段
        field, err_txt = DataConsolidation.list_col(configure['localhost'], configure['port'], configure['username'],
                                                    configure['password'], configure['database'], table_name)
        if err_txt:
            return None, "连接数据库表失败"
        data = {
            "list": list,
            "rows": nrows - 1,
            "table": table_name,
            "field": field
        }
        return data, None

    @staticmethod
    def data_migrate(file_path, configure, export_field, old_table_id, new_table_id):
        configure = json.loads(configure)  # 连接数据库配置
        try:
            target_db = pymysql.connect(
                host=configure['localhost'],
                port=int(configure['port']),
                user=configure['username'],
                password=configure['password'],
                db=configure['database'],
                charset="utf8",
            )
        except Exception as err:
            return None, "目标数据库连接失败"
        conn = target_db.cursor()
        where = "id = " + new_table_id
        table_name_sql = "SELECT `table_name` FROM migrate_platform_table WHERE {};".format(where)
        conn.execute(table_name_sql)
        table_name = conn.fetchone()
        data = DataExeclService.excl_import(json.dumps(configure), table_name[0], file_path)
        print(data)
        import_data = data[0]["list"]
        # 连接目标数据库
        # try:
        num = 0
        for dict in import_data:
            # row = tuple(i)
            if "id" in dict.keys():
                old_id = dict.pop("id")  # 弹出id 返回旧表主键id
                field = export_field.lstrip("id,")  # 去除首部id
            row = tuple(dict.values())  # 字典转元组
            sql = "INSERT INTO `{}` ({}) VALUES {};".format(table_name[0], field, row)
            print(sql)
            sql = sql.replace("''", "NULL").replace("''", "NULL")  # 处理空数据
            sql = sql.replace("'{}'", "NULL").replace("'{}'", "NULL")  # 处理空json
            conn.execute(sql)
            new_id = conn.lastrowid  # 返回新表主键id
            now = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time()))  # 获取当前时间
            # 迁移映射表数据
            migrate_old_to_new_data = {
                'old_table_id': old_table_id,
                'new_table_id': new_table_id,
                'old_data_id': old_id,
                'new_data_id': new_id,
                'created_time': now
            }
            migrate_old_to_new_data = tuple(migrate_old_to_new_data.values())  #
            # 迁移映射表数据插入
            migrate_old_to_new_field = "old_table_id,new_table_id,old_data_id,new_data_id,created_time"
            old_new_sql = "INSERT INTO `{}` ({}) VALUES {};".format("migrate_old_to_new", migrate_old_to_new_field,
                                                                    migrate_old_to_new_data)
            # print(old_new_sql)
            conn.execute(old_new_sql)

            num = num + 1
        # except Exception as e:
        #     return None, str(e)
        target_db.commit()
        conn.close()
        data = {
            "rows": num
        }
        return data, None

    @staticmethod
    def data_cover(file_path, configure, table_name, cover_where, cover_field):
        configure = json.loads(configure)  # 连接数据库配置
        try:
            target_db = pymysql.connect(
                host=configure['localhost'],
                port=int(configure['port']),
                user=configure['username'],
                password=configure['password'],
                db=configure['database'],
                charset="utf8",
            )
        except Exception as err:
            return None, "目标数据库连接失败"
        conn = target_db.cursor()
        readboot = xlrd.open_workbook(file_path)
        sheet = readboot.sheet_by_index(0)
        # # 获取excel的行和列
        nrows = sheet.nrows  # 行
        ncols = sheet.ncols  # 列
        first_row_values = sheet.row_values(0)  # 第一行数据
        list = []
        try:
            num = 1
            for row_num in range(1, nrows):
                row_values = sheet.row_values(row_num)
                if row_values:
                    str_obj = {}
                for i in range(len(first_row_values)):
                    ctype = sheet.cell(num, i).ctype
                    cell = sheet.cell_value(num, i)
                    if ctype == 2 and cell % 1 == 0.0:  # ctype为2且为浮点
                        cell = int(cell)  # 浮点转成整型
                        cell = str(cell)  # 转成整型后再转成字符串，如果想要整型就去掉该行
                    elif ctype == 3:
                        date = datetime(*xldate_as_tuple(cell, 0))
                        cell = date.strftime('%Y/%m/%d %H:%M:%S')
                    elif ctype == 4:
                        cell = True if cell == 1 else False
                    str_obj[first_row_values[i]] = cell
                list.append(str_obj)
                num = num + 1

            # del list[0]
            success_num = 0
            for dict in list:
                where = cover_where + "=" + "'" + dict[cover_where] + "'"
                select_sql = "SELECT * FROM {} WHERE {};".format(table_name, where)
                select_sql = select_sql.replace("''", "NULL").replace("''", "NULL")  # 处理空数据
                select_sql = select_sql.replace("'{}'", "NULL").replace("'{}'", "NULL")  # 处理空json
                conn.execute(select_sql)
                select_data = conn.fetchone()
                # print(select_sql)
                if select_data:
                    li = []
                    for i in cover_field.split(","):
                        if len(dict[i]) > 0:
                            update = i + "=" + "'" + dict[i] + "'"
                        else:
                            update = i + "=" + "''"
                        li.append(update)
                    str1 = ','.join(li)
                    sql = "UPDATE `{}` SET {} WHERE {};".format(table_name, str1, where)
                    sql = sql.replace("''", "NULL").replace("''", "NULL")  # 处理空数据
                    sql = sql.replace("'{}'", "NULL").replace("'{}'", "NULL")  # 处理空json
                    conn.execute(sql)
                    success_num = success_num + 1

        except Exception as e:
            return None, e
        target_db.commit()
        conn.close()
        data = {
            "rows": success_num
        }
        return data, None

    """
    @staticmethod
    def data_cover2(file_path, configure, table_name, cover_where, cover_field):
        configure = json.loads(configure)  # 连接数据库配置
        try:
            target_db = pymysql.connect(
                host=configure['localhost'],
                port=int(configure['port']),
                user=configure['username'],
                password=configure['password'],
                db=configure['database'],
                charset="utf8",
            )
        except Exception as err:
            return None, "目标数据库连接失败"
        conn = target_db.cursor()
        readboot = xlrd.open_workbook(file_path)
        sheet = readboot.sheet_by_index(0)
        # # 获取excel的行和列
        nrows = sheet.nrows  # 行
        ncols = sheet.ncols  # 列
        first_row_values = sheet.row_values(0)  # 第一行数据
        list = []
        # try:
        num = 1
        for row_num in range(1, nrows):
            row_values = sheet.row_values(row_num)
            if row_values:
                str_obj = {}
            for i in range(len(first_row_values)):
                ctype = sheet.cell(num, i).ctype
                cell = sheet.cell_value(num, i)
                if ctype == 2 and cell % 1 == 0.0:  # ctype为2且为浮点
                    cell = int(cell)  # 浮点转成整型
                    cell = str(cell)  # 转成整型后再转成字符串，如果想要整型就去掉该行
                elif ctype == 3:
                    date = datetime(*xldate_as_tuple(cell, 0))
                    cell = date.strftime('%Y/%m/%d %H:%M:%S')
                elif ctype == 4:
                    cell = True if cell == 1 else False
                str_obj[first_row_values[i]] = cell
            list.append(str_obj)
            num = num + 1

        success_num = 0
        for dict in list:
            seller_id = 0
            warehouse_id = 0
            goods_id = 0
            stock_id = 0
            sellers_sql = "SELECT id FROM sellers WHERE code = '{}';".format(dict['seller_id'])
            sellers_sql = sellers_sql.replace("''", "NULL").replace("''", "NULL")  # 处理空数据
            sellers_sql = sellers_sql.replace("'{}'", "NULL").replace("'{}'", "NULL")  # 处理空json
            conn.execute(sellers_sql)
            sellers_data = conn.fetchone()

            warehouse_sql = "SELECT id FROM warehouses WHERE code = '{}';".format(dict['warehouse_id'])
            warehouse_sql = warehouse_sql.replace("''", "NULL").replace("''", "NULL")  # 处理空数据
            warehouse_sql = warehouse_sql.replace("'{}'", "NULL").replace("'{}'", "NULL")  # 处理空json
            conn.execute(warehouse_sql)
            warehouse_data = conn.fetchone()

            brand_product_code = dict['brand_code'] + dict['product_code']
            goods_sql = "SELECT id FROM goods WHERE brand_product_code = '{}';".format(brand_product_code)
            goods_sql = goods_sql.replace("''", "NULL").replace("''", "NULL")  # 处理空数据
            goods_sql = goods_sql.replace("'{}'", "NULL").replace("'{}'", "NULL")  # 处理空json
            conn.execute(goods_sql)
            goods_data = conn.fetchone()

            if sellers_data:
                seller_id = sellers_data[0]
            if warehouse_data:
                warehouse_id = warehouse_data[0]
            if goods_data:
                goods_id = goods_data[0]
            if goods_data and warehouse_data:
                stocks_sql = "SELECT id FROM warehouse_good_stocks WHERE warehouse_id = '{}'and good_id = '{}';".format(
                    warehouse_id, goods_id)
                stocks_sql = stocks_sql.replace("''", "NULL").replace("''", "NULL")  # 处理空数据
                stocks_sql = stocks_sql.replace("'{}'", "NULL").replace("'{}'", "NULL")  # 处理空json
                conn.execute(stocks_sql)
                stocks_data = conn.fetchone()
                if stocks_data:
                    stock_id = stocks_data[0]

            num = re.sub(' ', "", re.sub(':', "", re.sub(r'/', "", dict['created_at'])))

            while True:
                number = ''.join(str(x) for x in random.sample(range(10), 4))
                number1 = ''.join(str(x) for x in random.sample(range(10), 4))
                if not number.startswith('0') and not number1.startswith('0'):
                    break
            tz = pytz.timezone('Asia/Shanghai')
            now_time = timezone.now().astimezone(tz=tz).strftime("%Y-%m-%d %H:%M:%S")
            order_data = {
                'seller_id': seller_id,
                'warehouse_id': warehouse_id,
                'number': "S" + num + number,
                "out_number": dict['out_number'],
                'default_amount': abs(float(dict['default_unit_price']) * int(dict['count'])),
                'amount': abs(float(dict['amount'])),
                'purchasing_amount': abs(float(dict['purchasing_amount']) * int(dict['count'])),
                'checkout_status': 2,
                # 'status': dict['status'],
                'status': dict['status'],
                'finish_time': dict['finish_time'],
                'created_at': dict['created_at'],
            }
            order_data = tuple(order_data.values())
            old_new_sql = "INSERT INTO `{}` ({}) VALUES {};".format("sale_orders",
                                                                    "seller_id,warehouse_id,number,out_number,default_amount,amount,purchasing_amount,checkout_status,status,finish_time,created_at",
                                                                    order_data)

            conn.execute(old_new_sql)

            order_lists_data = {
                'sale_order_id': conn.lastrowid,
                'number': "ZS" + num + number,
                "sale_label": '正品',
                'count': abs(int(dict['count'])),
                'default_unit_price': abs(float(dict['default_unit_price'])),
                'unit_price': abs(float(dict['default_unit_price']) + float(dict['unit_price'])),
                'default_amount': abs(float(dict['default_unit_price']) * int(dict['count'])),
                'amount': abs(float(dict['amount'])),
                'purchasing_amount': abs(float(dict['purchasing_amount']) * int(dict['count'])),
                'invoice_status': 1,
                'settlement_status': 1,
                "stock_id": stock_id,
                'created_at': dict['created_at'],
            }
            order_lists_data = tuple(order_lists_data.values())
            order_lists_sql = "INSERT INTO `{}` ({}) VALUES {};".format("sale_order_lists",
                                                                        "sale_order_id,number,sale_label,count,default_unit_price,unit_price,default_amount,amount,purchasing_amount,invoice_status,settlement_status,stock_id,created_at",
                                                                        order_lists_data)

            conn.execute(order_lists_sql)
            success_num = success_num + 1
        target_db.commit()
        conn.close()
        data = {
            "rows": success_num
        }
        return data, None """



# encoding: utf-8
"""
@project: djangoModel->synchronous_mysql_synchronous
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 使用pandas，数据库同步脚本
@created_time: 2023/5/18 17:47
"""
import json
import os
import time

from django.core.management.base import BaseCommand, no_translations
from pandas import DataFrame
import pandas as pd
import pymysql
import sqlalchemy
from sqlalchemy import create_engine

from config.config import JConfig
from xj_migrate.utils.custom_tool import format_params_handle, write_to_log, force_transform_type


# 迁移配置协议结构：
# [
#     {
#         "source_table": "flow_flow",
#         "target_table": "flow_flow2",
#         "primary_key": "id",
#         "filter_fields": [
#         ],
#         "remove_fields": [
#         ],
#         "alice_fields": {
#             "module_name": "module_name2"
#         },
#         # note 追加模式启用 relate_other_table_key
#         "relate_other_table_key": [
#             {
#                 "source_table": "flow_flow",
#                 "source_field": "flow_flow2",
#                 "foreign_key": "id"
#             }
#         ],
#         # note 后面面扩展功能实现数据清洗，然后再入库。
#         "data_cleaning": []
#     }
# ]

# 自定义异常类，记录异常，并打印错误信息
class MigrateError(Exception):
    def __init__(self, err_msg=None, *args, catch_err=None, **kwargs):
        super().__init__(self)
        self.err_msg = err_msg or str(catch_err)
        self.catch_err = catch_err

    def __str__(self):
        if self.catch_err:
            write_to_log(prefix="迁移异常", err_obj=self.catch_err)
        else:
            write_to_log(prefix="迁移异常", content=self.err_msg)
        err_msg = self.err_msg or str(self.catch_err)
        return err_msg


class Command(BaseCommand):
    # note 注意不同结构的表进行同步时候，需要设计好迁移的表的执行顺序、字段映射、迁移表映射。还有外键冲突，
    # 报错 TypeError: __init__() got multiple values for argument 'schema' ===>>> 处理方案： 更新依赖包：sqlalchemy==1.4.46，pandas==1.3.5
    # 报错：sqlalchemy.exc.OperationalError: (pymysql.err.OperationalError) (1054, "Unknown column 'index' in 'field list'") ===>>> 处理：index=False

    help = "同步数据库中配置表的结构和数据"
    from_engine = None  # 迁移源数据库配置
    to_engines = []  # 被迁移的数据配置
    migrate_configure = {}  # 迁移的表映射
    append_table_id_mapping = {}  # 追加模式的时候字段冲突，主键ID映射。其他关联的表获自行获取修改关联ID。
    __conflict_record = {}

    def get_conflict_record(self, *args, table_name=None, conflict_key=None, **kwargs):
        # 获取冲突记录
        if table_name is None:
            return self.__conflict_record
        elif not table_name is None and conflict_key is None:
            return self.__conflict_record.get(table_name, {})
        elif not table_name is None and not conflict_key is None:
            return self.__conflict_record.get(table_name, {}).get(conflict_key, None)
        return self.__conflict_record

    def set_conflict_record(self, *args, table_name=None, conflict_key=None, conflict_value=None, **kwargs):
        # 创建冲突映射
        if not self.__conflict_record.get(table_name):
            self.__conflict_record[table_name] = {}
        self.__conflict_record[table_name][conflict_key] = conflict_value
        # 同步到本地文件中
        self.conflict_record_fp.seek(0)
        self.conflict_record_fp.truncate()
        json.dump(self.__conflict_record, self.conflict_record_fp)

    def add_arguments(self, parser):
        parser.add_argument('--type',
                            dest='type',
                            default="replace",
                            help='迁移数据库，采用追加模式还是替换模式')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.config = JConfig()
        today = time.strftime("%Y-%m-%d", time.localtime(time.time()))
        save_path = self.config.get_base_dir() + "/resource_files/backup_files/" + today + '/'
        # ============ section 数据库连接相关 start===================
        self.from_engine = None
        self.to_engines = []
        self.from_db_map = self.config.get_list_by_cut(
            section="xj_migrate",
            option="migrate_from_db",
            map_key_list=["host", "user", "password", "database", "port"],
            cut_char=","
        )
        self.migrate_to_dbs = self.config.parse_lines_tuple(
            section="xj_migrate",
            option="migrate_to_dbs",
            cut_char=",",
            map_key_list=["host", "user", "password", "database", "port"]
        )
        # 来源数据库创建连接
        self.from_engine, err = self.create_connect(self.from_db_map, use_create_engine=True)
        if err:
            raise MigrateError("无法连接迁移源")
        # 同步的数据库创建连接
        for i in self.migrate_to_dbs:
            conn, err = self.create_connect(i, use_create_engine=True)
            if err:
                raise MigrateError("无法连接目标源")
            self.to_engines.append(conn)
        # ============ section 数据库连接相关 end  ===================

        # 迁移配置
        self.migrate_configure = self.config.load_json_configure("migrate_configure.json", default=[])
        self.target_table_to_primary_key = {i["target_table"]: i["primary_key"] for i in self.migrate_configure}
        self.migrate_configure = {i["source_table"]: i for i in self.migrate_configure}
        self.migrate_tables = self.config.get_list_by_cut(section="xj_migrate", option="migrate_tables", cut_char=",")

        # 冲突记录
        conflict_record_file_name = "conflict_record_" + str(int(time.time())) + ".json"
        self.conflict_record_fp = open(save_path + conflict_record_file_name, mode="w", encoding="utf-8")
        self.__conflict_record = {}

    @staticmethod
    def create_connect(config_dict: dict = None, use_sqlalchemy: bool = True, **kwargs):
        """
        构建数据库连接
        :param config_dict: 数据库参数
        :param use_sqlalchemy: 是否使用sqlalchemy引擎
        :return: data,err
        """
        # ================== section 获取基础参数 start ===============================
        config_dict, is_pass = force_transform_type(variable=config_dict, var_type="only_dict", default={})
        kwargs, is_pass = force_transform_type(variable=kwargs, var_type="only_dict", default={})
        config_dict.update(kwargs)
        host = config_dict.get("host", None)
        user = config_dict.get("user", None)
        password = config_dict.get("password", None)
        database = config_dict.get("database", None)
        port, is_pass = force_transform_type(variable=config_dict.get("port", None), var_type="int")
        use_sqlalchemy, is_pass = force_transform_type(variable=use_sqlalchemy, var_type="bool", default=False)
        if not host or not user or not password or not port:
            return None, "参数错误"
        # ================== section 获取基础参数 end ===============================
        try:
            # 使用sqlalchemy引擎
            if use_sqlalchemy:
                # 注意由于pandas兼容问题，写入的时候需要使用
                mysql_client_str = 'mysql+pymysql://' + str(user) + ':' + str(password) + '@' + str(host) + ':' + str(port) + '/' + str(database)
                return create_engine(mysql_client_str), None
            else:
                # 直接使用pymysql
                config_dict["port"], is_pass = force_transform_type(variable=config_dict.get("port", 3306), var_type="int", default=3306)
                return pymysql.connect(host=host, user=user, password=password, database=database, port=port), None
        except Exception as e:
            return None, str(e)

    @staticmethod
    def insert_by_sql(table, conn, keys, insert_tuple):
        """
        执行sql插入数据
        :param table: 表对象
        :param conn: 连接对象
        :param keys: 插入字段列表
        :param data_iter: 遍历数据
        :return: code ,err_msg
        """
        # 插入数据keys
        insert_keys_str = ""
        for k in keys:
            insert_keys_str = insert_keys_str + ("" if not insert_keys_str else ",") + k
        # 拼接values
        insert_value_str = ""
        for k in keys:
            insert_value_str = insert_value_str + ("" if not insert_value_str else ",") + "%s"
        # 拼接sql
        sql = """INSERT INTO `{TABLE_NAME}` ({INSERT_KEYS}) VALUES({INSERT_VALUES});""".format(
            TABLE_NAME=table.name,
            INSERT_KEYS=insert_keys_str,
            INSERT_VALUES=insert_value_str,
        )
        # 创建数据库游标对象
        dbapi_conn = conn.connection
        with dbapi_conn.cursor() as cursor:
            try:
                cursor.execute(sql, insert_tuple)  # 防止sql注入，使用占位替换方式
                insert_id = dbapi_conn.insert_id()
                return 0, None, insert_id
            except Exception as io_err:
                code, msg = io_err.args
                return code, msg, None

    @staticmethod
    def table_fields_types(engine, table_name: str = None):
        table = sqlalchemy.Table(table_name, sqlalchemy.MetaData(), autoload=True, autoload_with=engine)
        return {k: v.type for k, v in table.c.items()}

    def data_backup(self, pf, table_name="", save_type="to_json"):
        """在同步之前，把目标表做一次备份"""
        if not save_type in save_type:
            return False, "不是有效的保存方式"
        today = time.strftime("%Y-%m-%d", time.localtime(time.time()))
        save_path = self.config.get_base_dir() + "/resource_files/backup_files/" + today + '/'
        if not os.path.exists(save_path):
            os.makedirs(save_path)
        save_name = table_name + "-" + today + "-backup"
        save_path = save_path + save_name
        if save_type == "to_json":
            pf.to_json(save_path + ".json")
        elif save_type == "to_csv":
            pf.to_csv(save_path + ".csv")
        elif save_type == "to_excel":
            pf.to_excel(save_path + ".xlsx")

    def replace_insert(self, df: DataFrame, migrate_table, to_engine, fields_dtype, source_table, **kwargs):
        """替换默认插入"""
        df.to_sql(
            name=migrate_table,
            con=to_engine,
            if_exists="replace",
            dtype=fields_dtype,
            index=False
        )
        # note 替换模式会覆盖掉原有的主键ID，所以需要重新加回来。 暂时pandas还没有更好的解决办法
        primary_key = self.migrate_configure.get(source_table, {}).get("primary_key", "id")
        con = to_engine.connect()
        con.execute("""ALTER TABLE `{}`
        MODIFY COLUMN `{}` bigint(20) NOT NULL AUTO_INCREMENT FIRST,
        ADD PRIMARY KEY (`{}`);""".format(migrate_table, primary_key, primary_key))

    def append_insert(self, df: DataFrame, migrate_table, to_engine, fields_mapping, source_table, **kwargs):
        """追加模式插入"""
        # note 保存之前处理外键关联的问题，所以设计好导入的顺序。先导入的结果会影响后面的结果
        df.to_sql(
            name=migrate_table,
            con=to_engine,
            if_exists="append",
            dtype=fields_mapping,
            index=False,
            method=self.append_insert_callback
        )

    def append_insert_callback(self, table, conn, keys, data_iter, **kwargs):
        """
        Pandas中to_sql方法的回调函数
        :param table:Pandas的table
        :param conn:数据库驱动连接对象
        :param keys:要存入的字段名
        :param data_iter:DataFrame对象也就是数据迭代器
        :return:
        """
        primary_key = self.target_table_to_primary_key.get(str(table.name), "id")
        for item in data_iter:
            code, err_msg, last_insert_id = self.insert_by_sql(table, conn, keys, item)
            value_map = {k: v for k, v in zip(keys, item)}
            if code == 1062:
                # 主键冲突，去除主键，建立冲突映射
                old_primary_key = value_map.pop(primary_key, None)
                code, msg, last_insert_id = self.insert_by_sql(table, conn, list(value_map.keys()), list(value_map.values()))
                self.set_conflict_record(table_name=table.name, conflict_key=old_primary_key, conflict_value=last_insert_id)

    @no_translations
    def handle(self, *args, **options):
        # note 业务逻辑入口
        migrate_type = options["type"] if options.get("type") in ["replace", "append"] else "replace"
        if not self.migrate_tables:
            raise MigrateError("没有找到需要迁移的表")

        for table_name in self.migrate_tables:
            # 获取对应表的配置信息
            current_table_configure = self.migrate_configure.get(table_name, {})
            migrate_table = current_table_configure.get("target_table", table_name)
            # ============ section 获取源数据 start ===================
            source_pf = pd.read_sql("SELECT * FROM `" + table_name + "`", self.from_engine)
            from_table_fields = self.table_fields_types(engine=self.from_engine, table_name=table_name)
            # ============ section 获取源数据 end   ===================
            for to_engine in self.to_engines:
                target_df = source_pf
                # ============ section 迁移之前先对目标表数据进行备份 start ===================
                try:
                    copy_df = pd.read_sql("SELECT * FROM `" + migrate_table + "`", to_engine)
                    self.data_backup(copy_df, table_name=migrate_table)
                except sqlalchemy.exc.ProgrammingError:
                    pass
                except Exception as e:
                    raise MigrateError(catch_err=e)
                # ============ section 迁移之前先对目标表数据进行备份 end   ===================

                # ============ section 执行字段配置 start ===================
                # 过滤字段
                filter_fields = current_table_configure.get("filter_fields")
                if filter_fields and isinstance(filter_fields, list):
                    target_df = target_df.loc[:, filter_fields]
                # 移除字段
                remove_fields = current_table_configure.get("remove_fields")
                if remove_fields and isinstance(remove_fields, list):
                    target_df = target_df.drop(columns=remove_fields, inplace=False)
                # 别名替换
                alice_fields = current_table_configure.get("alice_fields", {})
                if alice_fields:
                    target_df = target_df.rename(columns=format_params_handle(
                        param_dict=alice_fields,
                        filter_filed_list=target_df.columns.to_list(),
                        alias_dict=alice_fields
                    ))
                # 冲突处理
                # 追加模式字段冲突处理，修改因为外键冲突受影响的数据。如：a表 b_id是b主键ID。b先迁移，但是迁移过程中主键冲突，变成了插入。得到结果： b表 ID 5 ==>> 10 ==>> a表 b_id 修改成10
                relate_keys = current_table_configure.get("relate_other_table_key", [])
                for i in relate_keys:
                    replace_params = self.get_conflict_record(table_name=i["target_table"])
                    if replace_params:
                        target_df[i["foreign_key"]].replace(replace_params, inplace=True)
                # ============ section 执行字段配置 start ===================

                # note 开始迁移
                if migrate_type == "replace":
                    self.replace_insert(
                        df=target_df,
                        migrate_table=migrate_table,
                        to_engine=to_engine,
                        fields_dtype=from_table_fields,
                        source_table=table_name
                    )
                else:
                    self.append_insert(target_df, migrate_table, to_engine, from_table_fields, table_name)

    def __del__(self):
        """迁移完成，释放内存"""
        self.conflict_record_fp.close()



from ..models import MigratePlatformTable


class MigratePlatfromService:

    @staticmethod
    def get():
        currencies = MigratePlatformTable.objects.all()

        return list(currencies.values('id', 'plaform_id', 'table_name')),None

    @staticmethod
    def post(params):
        plaform_id = params.get('plaform_id', '')
        table_name = params.get('table_name', '')
        pay_mode_set = MigratePlatformTable.objects.filter(table_name=table_name, plaform_id=plaform_id).first()
        if pay_mode_set is not None:
            return None, "数据已存在"
        try:
            MigratePlatformTable.objects.create(**params)
            return None, None
        except Exception as e:
            return None, "参数配置错误：" + str(e)
